<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Core
{
    use Interfaces\Enginepart;
    use \XTemplate\Engine;
    use \ArrayAccess;
    
    /**
     * Configuration class. It loads the configuration from an XML file and searches for configuration adapters which
     * can interpret the config entries. 
     * 
     * @author      Tobias Pohlen
     * @package     xtemplate
     * @version     $Id$ 
     */
    class Config extends EnginePartImpl implements ArrayAccess
    {
        /**
         * Inheritanced config objects
         * @var array
         */
        private $arrInheritances = array();
        /**
         * config entries
         * @var array
         */
        private $arrConfigEntries = array();
        /**
         * Registered config interpretors
         * @var string
         */
        private $arrInterpretors = array(
            "interpretors" => "XTemplate\\Core\\ConfigInterpretors\\Interpretors"
        );
        
        /**
         * Creates a new config object
         * 
         * @param       mixed               $strConfigFile      Path to the xml config file or an instance of 
         *                                                      XTemplate\Core\XML or an XML string
         * @param       XTemplate\Engine    $objEngine          XTemplate engine instance
         * @throws      XTemplate\Exceptions\CoreException
         */
        public function __construct($varXML, Engine $objEngine = null)
        {
            $this->setEngineInstance($objEngine);
            $objXML = XML::getInstance($varXML);

            // Does the config have the correct xml-tag name?
            $this->parseConfig($objXML);
        }
        
        /**
         * Parses the template
         */
        protected function parseConfig(XML $objXML)
        {
            // Is there a domain attribute?
            if ($objXML->getAttribute("domain") !== null)
            {
                // Do the domains match?
                if ($objXML->getAttribute("domain") !== Engine::getDomain())
                {
                    // This config file must not be parsed
                    return;
                }
            }
            
            // Is there an extending config file?
            if ($objXML->getAttribute("extends") !== null)
            {
                // There is
                // Create the new config object
                $objConfig = new Config($objXML->getAttribute("extends"), $this->getEngineInstance());
                $this->extend($objConfig);
            }
            
            // Is this a normal config or a domain config
            foreach ($objXML->children(Constants::XML_NS_URL) as $objEntry)
            {
                $strName = $objEntry->getName();
                
                // Is there an interpretor registrated for this config entry?
                if (isset($this->arrInterpretors[$strName]))
                {
                    $strClassName = $this->arrInterpretors[$strName];
                    
                    // Get the name of the config entry
                    $strEntryName = $objEntry->getAttribute("name");
                    
                    // Create the new interpretor instance
                    $objInterpretor = new $strClassName($objEntry, $this);
                    $this->addConfigEntry($strEntryName, $objInterpretor);
                }
            }
        }
        
        /**
         * Returns all interpretors
         * 
         * @return      array
         */
        public function getInterpretors()
        {
            return $this->arrInterpretors;
        }
        
        /**
         * Adds an interpretor
         * 
         * @param       string              $strName
         * @param       string              $strClassName
         */
        public function addInterpretor($strName, $strClassName)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            Params::string($strClassName, __CLASS__, __FUNCTION__);
            
            $this->arrInterpretors[$strName] = $strClassName;
        }
        
        /**
         * Adds an interpretor
         * 
         * @param       string              $strName        The name of the interpretor entry
         * @param       XTemplate\Core\ConfigInterpretor    $objInterpretor
         * @throws      XTemplate\Core\Exceptions\CoreException
         */
        public function addConfigEntry($strName, ConfigInterpretor $objInterpretor)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            
            if ($strName === "")
            {
                throw new Exceptions\CoreException("The interpretor name must not be empty.");
            }
            
            $this->arrConfigEntries[$strName] = $objInterpretor;
        }
        
        /**
         * Returns a config entry by its name
         * 
         * @param       string              $strName
         * @return      XTemplate\Core\ConfigInterpretor
         * @throws      XTemplate\Core\Exceptions\CoreException
         */
        public function getConfigEntry($strName)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            
            // Does the interpretor exist?
            if (!isset($this->arrConfigEntries[$strName]))
            {
                throw new Exceptions\CoreException("Entry '". $strName ."' does not exist.");
            }
            
            return $this->arrConfigEntries[$strName];
        }
        
        /**
         * Returns all config entries
         * 
         * @return      array
         */
        public function getAllConfigEntries()
        {
            return $this->arrConfigEntries;
        }
        
        /**
         * Extends this config form another one
         * 
         * @param       XTemplate\Core\Config $objConfig
         */
        public function extend(Config $objConfig)
        {
            $this->arrInheritances[] = $objConfig;
            $this->arrConfigEntries = array_merge($this->arrConfigEntries, $objConfig->getAllConfigEntries());
            $this->arrInterpretors = array_merge($this->arrInterpretors, $objConfig->getInterpretors());
        }
        
        /**
         * Returns all config entries of certain interpretor type
         * 
         * @param       string              $strName        The name of the interpretor e.g. x:config
         * @return      array
         */
        public function getConfigEntries($strName)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            
            // Is this name registrated
            if (!isset($this->arrInterpretors[$strName]))
            {
                return array();
            }
            
            // Get the classname
            $strClassName = $this->arrInterpretors[$strName];
            
            $arrResult = array();
            
            // walk through the interpretors and search for the right ones
            foreach ($this->arrConfigEntries as $objEntry)
            {
                if ($objEntry instanceof $strClassName)
                {
                    $arrResult[] = $objEntry;
                }
            }
            
            return $arrResult;
        }
        
        /**
         * Returns whether or not an interpretor exists
         * 
         * @see         ArrayAccess
         * @param       string              $strOffset
         * @return      mixed
         */
        public function offsetExists($strOffset)
        {
            return isset($this->arrConfigEntries[$strOffset]);
        }
        
        /**
         * Returns an interpretor value if it exists. If it doesn't it returns null
         * 
         * @see         ArrayAccess
         * @param       string              $strOffset
         * @return      XTemplate\Core\ConfigInterpretor
         * @throws      XTemplate\Core\Exceptions\CoreException
         */
        public function offsetGet($strOffset)
        {
            return $this->getConfigEntry($strOffset);
        }
        
        /**
         * Operator for Config::addInterpretor.
         * 
         * @see         ArrayAccess
         * @param       string              $strName
         * @param       XTemplate\Core\ConfigInterpretor    $objInterpretor
         * @throws      XTemplate\Core\Exceptions\CoreException
         */
        public function offsetSet($strName, $objInterpretor)
        {
            $this->addConfigEntry($strName, $objInterpretor);
        }
        
        /**
         * Removes a config entry. 
         * 
         * @see         ArrayAccess
         * @param       string              $strName
         */
        public function offsetUnset($strName)
        {
            if ($this->offsetExists($strName))
            {
                unset($this->arrConfigEntries[$strName]);
            }
        }
    }
}
?>