<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Core
{
    /**
     * This is a simple RAII implementation to work with files.
     * 
     * @author      Tobias Pohlen
     * @version     $Id$ 
     * @package     xtemplate
     */
    class File
    {
        /**
         * File handle
         * @var resource 
         */
        private $hdlFile = null;
        /**
         * Actual filename
         * @var string 
         */
        private $strFileName = "";
        /**
         * File mode
         * @var string 
         */
        private $strMode = "";
        
        /**
         * Creates a new file instance
         * 
         * @param       string                  $strFileName 
         * @throws      XTemplate\Exceptions\CoreException
         */
        public function __construct($strFileName, $strMode = "w")
        {
            Params::string($strFileName, __CLASS__, __FUNCTION__);
            Params::string($strMode, __CLASS__, __FUNCTION__);
            
            $this->strFileName = $strFileName;
            $this->strMode = $strMode;
            $this->allocate();
        }
        
        /**
         * Frees the resource again 
         */
        public function __destruct()
        {
            $this->free();
        }
        
        /**
         * Allocates a new resource handle 
         * 
         * @throws      XTemplate\Exceptions\CoreException
         */
        private function allocate()
        {
            $this->hdlFile = @fopen($this->strFileName, $this->strMode);
            if (!$this->hdlFile)
            {
                throw new Exceptions\CoreException("Could not open file '". $this->strFileName ."'.");
            }
        }
        /**
         * Frees the resource 
         */
        private function free()
        {
            @fclose($this->hdlFile);
        }
        
        /**
         * Writes to the file
         * 
         * @param       string                  $strContent
         * @throws      XTemplate\Exceptions\CoreException
         */
        public function write($strContent)
        {
            Params::string($strContent, __CLASS__, __FUNCTION__);
            
            if (!@fwrite($this->hdlFile, $strContent))
            {
                throw new Exceptions\CoreException("Could not write to file '". $this->strFileName ."'.");
            }
        }
        
        /**
         * Reads the content from the file. If length is -1 the current filesize is used.
         * 
         * @return      string  
         * @throws      XTemplate\Exceptions\CoreException
         */
        public function read($intLength = -1)
        {
            // If the file doesn't exist. Return an empty string.
            if (!$this->exists())
            {
                return "";
            }
            
            if ($intLength === -1 || $intLength > $this->filesize())
            {
                $intLength = @filesize($this->strFileName);
            }
            
            // If the file is empty, just return an empty string
            if ($intLength === 0)
            {
                return "";
            }
            
            $strContent = @fread($this->hdlFile, $intLength);
            if (!$strContent)
            {
                throw new Exceptions\CoreException("Could not read from file '". $this->strFileName ."'.");
            }
            
            return $strContent;
        }
        
        /**
         * Removes the current file and creates a new handle
         *  
         * @throws      XTemplate\Exceptions\CoreException
         */
        public function unlink()
        {
            $this->free();
            if (!unlink($this->strFileName))
            {
                throw new Exceptions\CoreException("Could not delete file '". $this->strFileName ."'.");
            }
            $this->allocate();
        }
        
        /**
         * Returns whether or not the file exists
         * 
         * @return      bool
         */
        public function exists()
        {
            return file_exists($this->strFileName);
        }
        
        /**
         * Resets the file pointer to a certain position.
         * 
         * @param       int             $intPosition
         */
        public function reset($intPosition = 0)
        {
            if (!@fseek($this->hdlFile, $intPosition))
            {
                throw new Exceptions\CoreException(
                        "Could move the file pointer to '". $intPosition ."' file '". $this->strFileName ."'.");
            }
        }
        
        /**
         * Returns the file size. Returns 0 if the file doesn't exist yet.
         * 
         * @return      int
         */
        public function filesize()
        {
            if (!$this->exists())
            {
                return 0;
            }
            return filesize($this->strFileName);
        }
    }
}
?>