<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Core
{
    /**
     * This is a helper class to check method and function parameters of their correct type. Arrays and objects should
     * directly be checked by typehinting. 
     * 
     * @author      Tobias Pohlen
     * @version     $Id$ 
     * @package     xtemplate
     */
    class Params
    {
        const ERROR_LEVEL = \E_USER_WARNING;
        
        /**
         * Returns the type of a variable as string
         * 
         * @param       mixed                   $varParam
         * @return      string
         */
        private static function getType($varParam)
        {
            if (is_string($varParam))
            {
                return "string";
            }
            
            else if (is_int($varParam))
            {
                return "integer";
            }
            
            else if (is_double($varParam))
            {
                return "double";
            }
            
            else if (is_float($varParam))
            {
                return "float";
            }
            
            else if (is_bool($varParam))
            {
                return "boolean";
            }
            
            else if (is_array($varParam))
            {
                return "array";
            }
            
            else if (is_object($varParam))
            {
                return "object";
            }
            
            else if ($varParam === null)
            {
                return "null";
            }
            else 
            {
                return "unkown";
            }
        }
        
        /**
         * Triggers the error 
         * 
         * @param       &mixed                  $varParam
         * @param       string                  $strExpected        What the method originally expected
         * @param       string                  $strClassName
         * @param       string                  $strMethodName
         */
        private static function triggerError($varParam, $strExpected, $strClassName, $strMethodName)
        {
            trigger_error(
                $strClassName ."->". $strMethodName ." expects a ". $strExpected .". ". self::getType($varParam) ." given.", 
                self::ERROR_LEVEL);
        }
        
        /**
         * Checks if a parameter is a string. Notice: FALSE is converted to an empty string (PHP internals).
         * 
         * @param       &string                 $varParam
         * @param       string                  $strClassName
         * @param       string                  $strMethodName
         */
        public static function string(&$varParam, $strClassName, $strMethodName)
        {
            // Everything except arrays and objects without a toString method can be converted to string
            if (!is_string($varParam) && 
                    (is_array($varParam) || (is_object($varParam) && !method_exists($varParam, "__toString"))))
            {
                self::triggerError($varParam, "string", $strClassName, $strMethodName);
            }
            
            $varParam = (string) $varParam;
        }
        
        /**
         * Checks if a parameter is an integer. 
         * 
         * @param       &int                    $varParam
         * @param       string                  $strClassName
         * @param       string                  $strMethodName
         */
        public static function int(&$varParam, $strClassName, $strMethodName)
        {
            // Only boolean can be converted into integer
            if (!is_int($varParam) && !is_bool($varParam))
            {
                self::triggerError($varParam, "integer", $strClassName, $strMethodName);
            }
            
            $varParam = (int) $varParam;
        }
        
        /**
         * Checks if a parameter is a double. 
         * 
         * @param       &double                 $varParam
         * @param       string                  $strClassName
         * @param       string                  $strMethodName
         */
        public static function double(&$varParam, $strClassName, $strMethodName)
        {
            // Boolean, integer and float can be converted to double
            if (!is_double($varParam) && (!is_int($varParam) && !is_float($varParam) && !is_bool($varParam)))
            {
                self::triggerError($varParam, "double", $strClassName, $strMethodName);
            }
            $varParam = (double) $varParam;
        }
        
        /**
         * Checks if a parameter is a boolean. 
         * 
         * @param       &float                  $varParam
         * @param       string                  $strClassName
         * @param       string                  $strMethodName
         */
        public static function bool(&$varParam, $strClassName, $strMethodName)
        {
            // Nothing can be implicitly converted into boolean
            if (!is_bool($varParam) && !is_int($varParam))
            {
                self::triggerError($varParam, "boolean", $strClassName, $strMethodName);
                $varParam = (bool) $varParam;
            }
            // Since it's quite common to use integers as booleans, this must be supportet, too
            // 0 = false
            // 1 = true
            elseif (is_int($varParam))
            {
                $varParam = (bool) $varParam;
            }
        }
        
        /**
         * Checks the type of a variable and throws an exception when the type is wrong
         * 
         * @param       mixed                   $varParams
         * @param       string                  $strTypeName
         * @return      mixed                   Corrected type
         * @throws      XTemplate\Exceptions\ModelException
         */
        public static function checkType($varParam, $strTypeName)
        {
            // Null is always ok
            if ($varParam === null)
            {
                return null;
            }
            
            switch ($strTypeName)
            {
                case "string":
                case "str":
                    self::string($varParam, "XTemplate\\Core\\Params", "checkType");
                    break;
                
                case "int":
                case "integer":
                    self::int($varParam, "XTemplate\\Core\\Params", "checkType");
                    break;
                        
                case "double":
                case "float":
                case "flt":
                case "dbl":
                    self::double($varParam, "XTemplate\\Core\\Params", "checkType");
                    break;
                        
                case "boolean":
                case "bool":
                case "bol":
                    self::bool($varParam, "XTemplate\\Core\\Params", "checkType");
                    break;
                
                case "array":
                case "arr":
                    if (!is_array($varParam))
                    {
                        $strType = self::getType($varParam);
                        throw new \XTemplate\Exceptions\ModelException("Model must be an array. "
                                ."'". $strType ."' given.");
                    }
                    break;
                
                default:
                    // Assume object
                    if ($varParam !== null && (!is_object($varParam) || !$varParam instanceof $strTypeName))
                    {
                        $strType = self::getType($varParam);
                        throw new \XTemplate\Exceptions\ModelException("Model must be an object. "
                                ."'". $strType ."' given.");
                    }
                    break;
            }
            
            return $varParam;
        }
    }
}
?>