<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate
{
    use \XTemplate\Core\Classes;
    use \XTemplate\Core\Constants;
    use \XTemplate\Exceptions\SectionizeException;
    use \XTemplate\Core\Params;
    
    /**
     * This is one of the most important classes of the whole system. Each template is splitted into sections. Each 
     * section is an own id scope and can contain even further sections. 
     * 
     * You can use the selector with the array operator.
     * 
     * @author      Tobias Pohlen
     * @version     $Id$ 
     * @package     xtemplate
     */
    class Section extends \XTemplate\Selectors\Parser implements \ArrayAccess, Interfaces\SubView, Interfaces\NodeEdit
    {
        const DOM_ELEMENT   = "DOMElement";
        const SECTION_ATTR  = "xsection";
        
        /**
         * Instance counter
         * @var int
         */
        private static $intInstances = 0;
        /**
         * Array with all original sections
         * @var array
         */
        private static $arrCache = array();
        
        /**
         * The main template object
         * @var XTemplate\ViewBase
         */
        private $objMainTemplate = null;
        /**
         * This is the section document
         * @var \DOMDocument
         */
        private $objSectionDocument = null;
        /**
         * The parent section
         * @var XTemplate\Section
         */
        private $objParent = null;
        /**
         * The origin element in the parent template
         * @var XTemplate\DOMElement
         */
        private $objOriginalElement = null;
        /**
         * The current rootnode of the section template
         * @var XTemplate\DOMElement
         */
        private $objDOMElement = null;
        /**
         * The spaceholder in the parent document
         * @var XTemplate\DOMElement
         */
        private $objSpaceHolder = null;
        /**
         * Included sections
         * @var array
         */
        private $arrSections = array();
        /**
         * Created section clones
         * @var array
         */
        private $arrClones = array();
        /**
         * The instance ID
         * @var int
         */
        private $intID = 0;
        /**
         * Assigned subview
         * @var array
         */
        private $arrSubViews = array();
        
        /**
         * Creates a new section
         * 
         * @param       XTemplate\DOMElement            $objElement
         * @param       XTemplate\ViewBase              $objView
         */
        public function __construct(DOMElement $objElement, ViewBase $objView, Section $objParent = null)
        {
            $this->objDOMElement    = $objElement;
            $this->objMainTemplate  = $objView;
            $this->objParent        = $objParent;
            $this->intID            = self::$intInstances;
            self::$intInstances++;
            
            // Get the document of the parent section
            $objParentSectionDoc = $this->getParentSectionDoc();
            
            try
            {
                // The original section DOM element has to be removed from the template to not get touched by any
                // selectors
                // Add a space holder instead.
                $this->objSpaceHolder = $objParentSectionDoc->createElement("spacerholder_". $this->intID);
                // Append the spaceholder
                $this->objDOMElement->parentNode->insertBefore($this->objSpaceHolder, $this->objDOMElement);
                // Remove the actual element from the template
                $this->objDOMElement->remove();
                
                // Remove the section attribute to avoid recursions
                $this->objDOMElement->removeAttribute(self::SECTION_ATTR);
                $this->objOriginalElement = $this->objDOMElement->cloneNode(true);

                // Create the new section document
                $this->objSectionDocument = new \DOMDocument(Constants::DOM_VERSION, Constants::DOM_CHARSET);
                $this->objSectionDocument->registerNodeClass(
                        self::DOM_ELEMENT, $objView->getEngineInstance()->getClassName(self::DOM_ELEMENT));

                // Now move the root element of the section from the parent template to the section template
                $this->objDOMElement = $this->objSectionDocument->importNode($this->objDOMElement, true);
                $this->objSectionDocument->appendChild($this->objDOMElement);
                
                // Each section needs an ID to by identified by the clones
                self::$arrCache[$this->intID] = $this;
                
                // Check for included sections
                $this->parseSections();
            }
            catch (\DOMException $objException)
            {
                throw new SectionizeException($objException->getMessage());
            }
            
            parent::__construct(
                    $this->objDOMElement, $this->objSectionDocument, $this->objMainTemplate->getEngineInstance());
        }
        
        /**
         * Finds and creates the included sections
         * 
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        private function parseSections()
        {
            $objEngine = $this->objMainTemplate->getEngineInstance();
            
            // Create a new XPath object to get all subsections. Cannot be overwritten
            $objXPath = new \DOMXPath($this->objSectionDocument);
            
            // Create the xpath string
            $strXPath = $this->objDOMElement->getNodePath() . 
                    "/descendant::*[@". self::SECTION_ATTR. "]/ancestor-or-self::*[@". 
                    self::SECTION_ATTR ."][position()=last()]";
            $objNodeSet = $objXPath->query($strXPath);
            
            if (!$objNodeSet)
            {
                throw new SectionizeException("Could not perform sectionize query.");
            }
            
            // The nodelist is connected to the document. So we move all nodes to an array to avoid problems while 
            // moving the nodes
            $intLength = $objNodeSet->length;
            $arrNodes = array();
            for ($i = 0; $i < $intLength; $i++)
            {
                $arrNodes[] = $objNodeSet->item($i);
            }
            
            foreach ($arrNodes as $objNode)
            {
                // Is there already a section with this name?
                if (isset($this->arrSections[$objNode->getAttribute(self::SECTION_ATTR)]))
                {
                    throw new SectionizeException("Duplicate section name '". 
                            $objNode->getAttribute(self::SECTION_ATTR) ."'.");
                }
                
                // Create a new section inside of this section
                $this->arrSections[$objNode->getAttribute(self::SECTION_ATTR)] = $objEngine->createInstance(
                        Classes::SECTION, 
                        $objNode, 
                        $this->objMainTemplate, 
                        $this);
            }
        }
        
        /**
         * Adds a section clone
         * 
         * @param       XTemplate\Section
         */
        public function addClone(Section $objSection)
        {
            $this->arrClones[] = $objSection;
        }
        
        /**
         * Sets the refering section dom element
         * 
         * @param       XTemplate\DOMElement        $objDOMElement
         */
        public function setDOMElement(DOMElement $objDOMElement)
        {
            $this->objDOMElement = $objDOMElement;
        }
        
        /**
         * Sets the parent section
         * 
         * @param       XTemplate\Section           $objParent
         */
        public function setParent(Section $objParent)
        {
            $this->objParent = $objParent;
        }
        
        /**
         * Returns a section 
         * 
         * @param       string              $strSectionName     The name of the section which shall be cloned
         * @return      XTemplate\Section
         * @throws      OutOfRangeException
         */
        public function getSection($strSectionName)
        {
            // No typecheck here
            $strSectionName = (string) $strSectionName;
            
            // Does this section exist?
            if (!isset($this->arrSections[$strSectionName]))
            {
                throw new \OutOfRangeException("Section '". $strSectionName ."' does not exist.");
            }
            
            return $this->arrSections[$strSectionName];
        }
        
        /**
         * Returns a section 
         * 
         * @param       string              $strSectionName     The name of the section which shall be cloned
         * @return      XTemplate\Section
         * @throws      OutOfRangeException
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        public function getSectionClone($strSectionName)
        {
            return clone $this->getSection($strSectionName);
        }

        /**
         * Clones the section
         * 
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        public function __clone()
        {
            try
            {
                // Add the clone
                self::$arrCache[$this->intID]->addClone($this);

                // Create a new section document
                $objView = $this->objMainTemplate;

                $this->objSectionDocument = new \DOMDocument(Constants::DOM_VERSION, Constants::DOM_CHARSET);
                $this->objSectionDocument->registerNodeClass(
                        self::DOM_ELEMENT, 
                        $objView->getEngineInstance()->getClassName(self::DOM_ELEMENT));

                // Create a clone of the root element
                $this->objDOMElement = $this->objOriginalElement->cloneNode(true);

                // Add the clone
                $this->objDOMElement = $this->objSectionDocument->importNode($this->objDOMElement, true);
                $this->objSectionDocument->appendChild($this->objDOMElement);

                // Clear the clone array
                $this->arrClones = array();
                // Clear the section array
                $this->arrSections = array();

                $this->setSelectorNode($this->objDOMElement);
                $this->setSelectorTemplate($this->objSectionDocument);

                // Reparese the included sections
                $this->parseSections();
            }
            catch (\DOMException $objException)
            {
                throw new SectionizeException($objException->getMessage());
            }
        }
        
        /**
         * Returns if there are element which match this selector
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @return      bool
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetExists($strSelector)
        {
            // No type check here to safe performance
            $strSelector = (string) $strSelector;
            return count($this->select($strSelector)) > 0;
        }
        
        /**
         * Sets the content of the matched elements
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @param       string              $strContent
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetSet($strSelector, $strContent)
        {
            // No type check here to safe performance
            $strSelector = (string) $strSelector;
            $strContent = (string) $strContent;
            
            $this->select($strSelector)->setContent($strContent);
        }
        
        /**
         * Returns the resultset of the selector
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @return      XTemplate\ElementList
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetGet($strSelector)
        {
            // No type check here to safe performance
            $strSelector = (string) $strSelector;
            return $this->select($strSelector);
        }
        
        /**
         * Removes the matching elements from the template
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetUnset($strSelector)
        {
            // No type check here to safe performance
            $strSelector = (string) $strSelector;
            $this->select($strSelector)->remove();
        }
        
        /**
         * Renders the section
         * 
         * @throws      XTemplate\Exceptions\RenderException
         */
        public function render()
        {
            // Render all section clones and add the dom elements
            foreach ($this->arrClones as $objSection)
            {
                $objSection->renderSection();
            }
            
            // Remove the spaceholder
            $this->objSpaceHolder->remove();
        }
        
        /**
         * Renders the actual section clone
         * 
         * @throws      XTemplate\Exceptions\RenderException
         */
        public function renderSection()
        {
            // Render the sub sections
            foreach ($this->arrSections as $objSection)
            {
                $objSection->render();
            }
            
            try 
            {
                // Insert the subviews
                $objSubViewTags = $this->select("xsub");
                
                foreach ($objSubViewTags as $objCurrentTag)
                {
                    // Get the name attribute
                    if ($objCurrentTag->getAttribute("name") === null)
                    {
                        throw new Exceptions\RenderException("You cannot insert a subview without a name.");
                    }
                    
                    // Assign the subview
                    $objAssignedSubViews = $this->getAssignedSubViews($objCurrentTag->getAttribute("name"));
                    
                    foreach ($objAssignedSubViews as $objCurrentSubView)
                    {
                        $objCurrentTag->appendBefore($objCurrentSubView);
                    }
                    
                    $objCurrentTag->remove();
                }
                
                $objNode = $this->getParentSectionDoc()->importNode($this->objDOMElement, true);
                $this->objSpaceHolder->parentNode->insertBefore($objNode, $this->objSpaceHolder);
            }
            catch (\DOMException $objException)
            {
                // We have to be shure to stick to the interface of the method
                throw new Exceptions\RenderException($objException);
            }
        }
        
        /**
         * Returns the parent section document
         * 
         * @return      \DOMDocument
         */
        private function getParentSectionDoc()
        {
            if ($this->objParent === null)
            {
                return $this->objMainTemplate;
            }
            return $this->objParent->getDocument();
        }
        
        /**
         * Returns the section document
         * 
         * @return      \DOMDocument
         */
        public function getDocument()
        {
            return $this->objSectionDocument;
        }
        
        /**
         * Assignes a subview
         * 
         * @param       string              $strName        The name of the subview
         * @param       XTemplate\View      $objDoc         The subview document
         */
        public function assignSubView($strName, \XTemplate\SubView $objDoc)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            
            // Is there already a storage for these subviews?
            if (!isset($this->arrSubViews[$strName]))
            {
                $this->arrSubViews[$strName] = new SubViewStorage();
            }
            $this->arrSubViews[$strName]->attach($objDoc);
        }
        
        /**
         * Returns the subviews assigned under a certain name. If there are no subviews an empty object storage is 
         * returned.
         * 
         * @param       string              $strName        The name of the subview
         * @return      XTemplate\SubViewStorage
         */
        public function getAssignedSubViews($strName)
        {
            // Are there any subviews?
            if (!isset($this->arrSubViews[$strName]))
            {
                return new SubViewStorage();
            }
            else
            {
                return $this->arrSubViews[$strName];
            }
        }
        
        /**
         * Appends the subviews to a certain element in the template. e.g.:
         * You add multiple subviews in your controller to the view under the name "content". Now you want to put these
         * views to the element with the ID content:
         * 
         * $this->attachSubViews("#content", "content");
         * 
         * @param       string              $strSelector    A CSS selector which finds the corresponding nodes or an
         *                                                  instance of XTemplate\Interfaces\NodeEdit.
         * @param       string              $strSubViewName The name of the subviews which shall be attached
         */
        public function attachSubViews($strSelector, $strSubViewName)
        {
            if ($strSelector instanceof Interfaces\NodeEdit)
            {
                $objMatchingNodes = $strSelector;
            }
            else
            {
                // Get the nodes which receive the subviews
                $objMatchingNodes = $this->select($strSelector);
            }
            
            // Get the assigned subviews
            $objSubViews = $this->getAssignedSubViews($strSubViewName);
            
            foreach ($objSubViews as $objSubView)
            {
                $objMatchingNodes->append($objSubView);
            }
        }
        /**
         * Sets the content of the nodes. If the node contains other nodes they will be removed. There are two common 
         * ways you can use this method:
         * 
         * XTemplate\ElementList:
         * $this["#selector"]->setContent("a string");
         * Sets the content of all nodes which have the attribute 'xid="selector"'. 
         * 
         * XTemplate\DOMElement
         * $objNode->setContent("a string");
         * 
         * The parameter are proved with XTemplate\Core\Params. If you add something which cannot be converted 
         * implicitly to a string. You get a E_USER_WARNING error. 
         * 
         * @param       string                $strContent
         * @return      XTemplate\Interfaces\NodeEdit
         */
        public function setContent($strContent)
        {
            return $this->objDOMElement->setContent($strContent);
        }
        
        /**
         * Returns the node content. If there is no node, an empty string will be returned.
         * 
         * @return      string
         * @see         XTemplate\Interfaces\NodeEdit
         */
        public function getContent()
        {
            return $this->objDOMElement->getContent();
        }
        
        /**
         * NOT SUPPORTED HERE.
         * 
         * @throws      XTemplate\Exceptions\NodeEditException
         */
        public function remove()
        {
            throw new Exceptions\NodeEditException("Remove is not supported by sections");
        }
        
        /**
         * Sets the value of an existing attribute with the name $strName or adds this attribute. The value is always
         * the string $strValue. The parameters are proved by XTemplate\Core\Params. If you pass something which cannot
         * be implicitly be converted into a string, you get a E_USER_WARNING error. 
         * 
         * @param       string                $strName
         * @param       string                $strValue
         * @return      XTemplate\Interfaces\NodeEdit
         */
        public function setAttribute($strName, $strValue)
        {
            return $this->objDOMElement->setAttribute($strName, $strValue);
        }
        
        /**
         * This method returns the value of the attribute with name $strName. The parameter is proved by 
         * XTemplate\Core\Params. If you pass something which cannot be converted implicitly into string, you get an 
         * E_USER_WARNING error. 
         * 
         * 
         * If the attribute which you requested does not exist, an XTemplate\Exceptions\NodeEditException is thrown.
         * 
         * @param       string                $strName
         * @return      string
         * @throws      XTemplate\Exceptions\NodeEditException
         */
        public function getAttribute($strName)
        {
            return $this->objDOMElement->getAttribute($strName);
        }
        
        /**
         * Adds a classname to the class attribute. The parameter is proved by 
         * XTemplate\Core\Params. If you pass something which cannot be converted implicitly into string, you get an 
         * E_USER_WARNING error. 
         * 
         * Before
         * <a xid="selector" class="bar" />
         * 
         * After
         * <a xid="selector" class="bar foo" />
         * 
         * @param       string                $strClass
         * @return      XTemplate\Interfaces\NodeEdit
         */
        public function addClass($strClass)
        {
            return $this->objDOMElement->addClass($strClass);
        }
        
        /**
         * Removes a class from the class attribute. The parameter is proved by 
         * XTemplate\Core\Params. If you pass something which cannot be converted implicitly into string, you get an 
         * E_USER_WARNING error. 
         * 
         * Before
         * <a xid="selector" class="bar foo" />
         * 
         * After
         * <a xid="selector" class="bar" />
         * 
         * @param       string                $strClass
         * @return      XTemplate\Interfaces\NodeEdit
         */
        public function removeClass($strClass)
        {
            return $this->objDOMElement->removeClass($strClass);
        }
        
        /**
         * NOT SUPPORTED HERE
         * 
         * @throws      XTemplate\Exceptions\NodeEditException
         */
        public function appendHTML($strHTML)
        {
            throw new Exceptions\NodeEditException("Not supported: appendHTML");
        }
        
        /**
         * Converts the current node or the first node of the set to a string. If there is no node, an empty string will
         * be returned. This method must not throw any exception. 
         * 
         * @return      string
         */
        public function __toString()
        {
            return $this->objDOMElement->__toString();
        }
        
        /**
         * Appends a child(-tree) to the current node. 
         * 
         * @param       DOMNode            $objElement  The node which shall be inserted. You can also pass a 
         *                                              XTemplate\SubView instance.
         * @return      XTemplate\Interfaces\NodeEdit
         * @throws      XTemplate\Exceptions\NodeEditException
         */
        public function append(\DOMNode $objElement)
        {
            return $this->objDOMElement->append($objElement);
        }
        
        /**
         * Let's you iterate over all matched nodes and perform a custom function on every node. The functions must be 
         * wraped in the Code\FuncWrapper class. 
         * 
         * @param       XTemplate\FuncWrapper  $objCallback
         * @return      XTemplate\Interfaces\NodeEdit
         */
        public function callback(\XTemplate\FuncWrapper $objCallback)
        {
            return $this->objDOMElement->callback($objCallback);
        }
        
        /**
         * NOT SUPPORTED HERE 
         * 
         * @throws      XTemplate\Exceptions\NodeEditException
         */
        public function appendBefore(\DOMNode $objElement, \DOMNode $objRefNode = null)
        {
            throw new Exceptions\NodeEditException("Not supported: appendBefore");
        }
        
        /**
         * Appends HTML but converts it into a DOM object before. The parameter string must be valid XML!
         * 
         * @param       string              $strXML     A valid XML string
         * @throws      XTemplate\Exceptions\NodeEditException
         * @return      XTemplate\Interfaces\NodeEdit
         */
        public function appendXML($strXML)
        {
            return $this->objDOMElement->appendXML($strXML);
        }
        
        /**
         * A abbreviation of the getSectionClone method. 
         * 
         * @param       string              $strSectionName     The name of the section which shall be cloned
         * @return      XTemplate\Section
         * @throws      OutOfRangeException
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        public function __invoke()
        {
            $this->checkRendered();
            $arrArgs = func_get_args();
            
            // Are the argument correct?
            if (count($arrArgs) < 1)
            {
                throw new Exceptions\SectionizeException("Missing argument: section name.");
            }
            
            // Get the section
            return $this->getSectionClone($arrArgs[0]);
        }
        
    }
}
?>