<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Selectors
{
    use \XTemplate\Core\Params;
    use \XTemplate\Core\Classes;
    use \XTemplate\Core\Helper;
    use \XTemplate\Selectors\Exceptions\SelectorException;
    use \XTemplate\Core\EnginePartImpl;
    use \DOMXPath;
    
    /**
     * This is an implementation of the XTemplate\Selector interface. The different selectors are configured in the
     * config fie. You can even create your own selector. The class parses the string selectors and creates the selector
     * model.
     * 
     * @author      Tobias Pohlen
     * @version     $Id$ 
     * @package     xtemplate
     */
    class Parser extends EnginePartImpl implements \XTemplate\Interfaces\Selector
    {
        /**
         * Regular expression for a name
         */
        const REGEX_NAME = "[A-Za-z_]{1}[A-Za-z0-9_\-()]*";
        /**
         * Regular expression for an argument
         */
        const REGEX_ARG = "(\"([^\"]*)\"|'([^']*)')";
        /**
         * RegEx names. They are short to safe performance
         */
        const NEW_SELECTOR          = "n";
        const TAGNAME               = "t";
        const PATH_SELECTOR         = "p";
        const PATH_SELECTOR2        = "q";
        const ATTRIBUTE_SELECTOR    = "a";
        const CONJUNCTION           = "c";
        
        /**
         * RegEx chars which have to be escaped
         * @var array
         */
        private static $arrRegExChars = array("|", "+", "{", "}", "[", "]", "*", "-", ".", "/", "^", '$');
        /**
         * RegEx chars which are escaped
         * @var array
         */
        private static $arrEscapedRegExChars = array("\|", "\+", "\{", "\}", "\[", "\]", "\*", "\-", "\.", "\/", "\^", 
            '\$');
        
        /**
         * The refering root node of the DOM
         * @var XTemplate\DOMElement
         */
        private $objSelectorNode = null;
        /**
         * The template object
         * @var \DOMDocument
         */
        private $objSelectorTemplate = null;
        /**
         * Path selectors
         * @var array
         */
        private $arrPathSelectors = array();
        /**
         * Attribute selectors
         * @var array
         */
        private $arrAttributeSelectors = array();
        /**
         * Selector conjunctions
         * @var array
         */
        private $arrSelectorConjunctions = array();
        /**
         * The regular expression for this selector
         * @var string
         */
        private $strRegEx = "";
        /**
         * The regular expression for the path selectors
         * @var string
         */
        private $strRegExPath = "";
        /**
         * The regular expression for the attribute selectors
         * @var string
         */
        private $strRegExAttribute = "";
        /**
         * The classname of the ElementList. (Could be overwritten)
         * @var string
         */
        private $strElementListClassName = "";
        /**
         * XPath instance
         * @var \DOMXPath
         */
        private $objXPath = null;
        
        /**
         * Returns an array of operators from the config definitions and takes that all characters which are used for 
         * regular expressions are escaped. This method is protected to perform unit tests on it. It's not part of the
         * actual class interface.
         * 
         * @param       array               $arrDefinitions
         * @return      array
         * @access      private
         */
        protected static function getOperators(array $arrDefinitions)
        {
            $arrOperators = array_keys($arrDefinitions);
            
            // Walk through the array and replace all important regex chars
            $intLength = sizeof($arrOperators);
            for ($i = 0; $i < $intLength; ++$i)
            {
                // Escape the reg ex chars
                $arrOperators[$i] = str_replace(self::$arrRegExChars, self::$arrEscapedRegExChars, $arrOperators[$i]);
            }
            
            return $arrOperators;
        }
        
        /**
         * Sets the select node
         * 
         * @param       \DOMElement         $objElement
         */
        public function setSelectorNode(\DOMElement $objElement)
        {
            $this->objSelectorNode = $objElement;
        }
        
        /**
         * Sets the select document
         * 
         * @param       \DOMDocument        $objDocument
         */
        protected function setSelectorTemplate(\DOMDocument $objDocument)
        {
            $this->objSelectorTemplate = $objDocument;
            $this->objXPath = new \DOMXPath($objDocument);
        }

        /**
         * Creates a new selector
         * 
         * @param       XTemplate\DOMElement        $objSelectorNode    The root node where the selector starts at
         * @param       \DOMDocument                $objView            The template object
         */
        public function __construct(
                \DOMElement $objSelectorNode, 
                \DOMDocument $objView, 
                \XTemplate\Engine $objEngine)
        {
            $this->setSelectorNode($objSelectorNode);
            $this->setSelectorTemplate($objView);
            
            // Copy the engine instance to have the correct config file
            $this->setEngineInstance($objEngine);
            
            // Get all selectors and their operators
            $arrEntries = $this->getEngineInstance()->getConfig()->getConfigEntries("selectors");
            
            foreach ($arrEntries as $objEntry)
            {
                // Import the defined selectors
                $this->importSelectors($objEntry);
            }
            
            // Create the regular expression for the selectors
            $this->createRegEx();
            
            $this->strElementListClassName = $this->getEngineInstance()->getClassName(Classes::ELEMENTLIST);
        }
        
        /**
         * Imports selectors from the configuration
         * 
         * @param       XTemplate\Core\ConfigInterpretors\Selectors $objSelectors
         */
        private function importSelectors(\XTemplate\Core\ConfigInterpretors\Selectors $objSelectors)
        {
            // Import the path selectors
            foreach ($objSelectors->getPathSelectors() as $strOperator => $strClassName)
            {
                $this->createSelectorInstance($this->arrPathSelectors, $strOperator, $strClassName);
            }
            
            // Import the attribute selectors
            foreach ($objSelectors->getAttributeSelectors() as $strOperator => $strClassName)
            {
                $this->createSelectorInstance($this->arrAttributeSelectors, $strOperator, $strClassName);
            }
            
            // Import the selector conjunctions
            foreach ($objSelectors->getSelectorConjunctions() as $strOperator => $strClassName)
            {
                $this->createSelectorInstance($this->arrSelectorConjunctions, $strOperator, $strClassName);
            }
        }
        
        /**
         * Create a new selector instance. This method is protected to perform unit tests on it. It's not part of the
         * actual class interface.
         * 
         * @param       string                  $strOperator
         * @param       string                  $strSelectorClass
         * @access      private
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        protected function createSelectorInstance(array &$arrStorage, $strOperator, $strClassName)
        {
            $arrStorage[$strOperator] = new $strClassName();
            
            if (!$arrStorage[$strOperator] instanceof \XTemplate\Selectors\Base)
            {
                throw new SelectorException("Invalid selector '". $strClassName ."'.");
            }
        }
        
        /**
         * Creates the regular expression
         */
        protected function createRegEx()
        {
            // Start with an optional tag name
            // The tag name can either be a name or a *
            $strTagName = "(".self::REGEX_NAME ."|\*)";
            
            // The path selectors can be stacked
            $arrPathOperators = self::getOperators($this->arrPathSelectors);
            $this->strRegExPath = "(". implode("|", $arrPathOperators) .")(". self::REGEX_NAME .")";
            
            // There can only be one attribute selector
            // Get all attribute operators
            $arrAttributeOperators = self::getOperators($this->arrAttributeSelectors);
            $strAttributeSelector = "\[(". self::REGEX_NAME . ")(". implode("|", $arrAttributeOperators) .")(". 
                    self::REGEX_ARG .")\]";
            $this->strRegExAttribute = "/". $strAttributeSelector ."/";
            
            // There can only be one conjunction between two selectors
            // Get all conjunctions
            $arrConjunctions = self::getOperators($this->arrSelectorConjunctions);
            
            // Please don't break that line, although it's abover the 120 chars
            $this->strRegEx = "/(?<". self::NEW_SELECTOR .">[ ]*,[ ]*)?(?<". self::TAGNAME .">". $strTagName .")?(?<". 
                    self::PATH_SELECTOR .">(". $this->strRegExPath .")*)(?<". self::ATTRIBUTE_SELECTOR .">". 
                    $strAttributeSelector .")?(?<". self::PATH_SELECTOR2 .">(". $this->strRegExPath .")*)". 
                    "[ ]*(?<". self::CONJUNCTION .">". "(". implode("|", $arrConjunctions) .")" .")?[ ]*/";
            
            $this->strRegExPath = "/(". $this->strRegExPath .")/";
        }
        
        /**
         * Creates the XPath models for a certain selector. This method is protected to perform unit tests on it. It's 
         * not part of the actual class interface.
         * 
         * @param       &string                 $strSelector            The CSS selector
         * @return      array                                           Array of selector models
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         * @access      private
         */
        protected function createXPath($strSelector)
        {
            // Array of all selector models
            $arrSelectorModels = array(
                array()
            );
            
            // The current position of the model stack
            $intPos = 0;
            $strLastConjunction = "";
            
            
            // Get the result of the regex
            $arrMatch = array();

            //$varResult = preg_match($this->strRegEx, $strSelector, $arrMatch, \PREG_OFFSET_CAPTURE, $intOffset);
            $varResult = preg_match_all($this->strRegEx, $strSelector, $arrMatch);

            $intLength = sizeof($arrMatch[0]) - 1;
            
            // Everything alright?
            if ($intLength === -1 || strlen($arrMatch[0][0]) === 0 || $varResult === 0 || $varResult === false)
            {
                // There is a syntax error
                throw new SelectorException(
                        "Invalid selector '". $strSelector ."'.");
            }


            $i = 0;
            while ($i < $intLength)
            {
                // Shall a new model be created?
                if ($arrMatch[self::NEW_SELECTOR][$i] !== "")
                {
                    // Start a new selector stack
                    $arrSelectorModels[] = array();
                    ++$intPos;
                }
                
                // Create the selector model
                $objModel = new SelectorModel;
                $arrSelectorModels[$intPos][] = $objModel;
                
                // Parse the tagname
                $this->parseTagName($arrMatch[self::TAGNAME][$i], $strLastConjunction, $objModel);
                
                if (isset($arrMatch[self::PATH_SELECTOR][$i]) && $arrMatch[self::PATH_SELECTOR][$i] !== "")
                {
                    // Parse the path selectors
                    $this->parsePathSelectors($arrMatch[self::PATH_SELECTOR][$i], $objModel);
                }
                
                if (isset($arrMatch[self::PATH_SELECTOR2][$i]) && $arrMatch[self::PATH_SELECTOR2][$i] !== "")
                {
                    // Parse the path selectors
                    $this->parsePathSelectors($arrMatch[self::PATH_SELECTOR2][$i], $objModel);
                }
                
                if (isset($arrMatch[self::ATTRIBUTE_SELECTOR][$i]) && $arrMatch[self::ATTRIBUTE_SELECTOR][$i] !== "")
                {
                    // Parse the attribute selectors
                    $this->parseAttributeSelectors($arrMatch[self::ATTRIBUTE_SELECTOR][$i], $objModel);
                }
                
                // The conjunction is passed to the next tag selector
                if (isset($arrMatch[self::CONJUNCTION][$i]))
                {
                    $strLastConjunction = $arrMatch[self::CONJUNCTION][$i];
                }
                else
                {
                    $strLastConjunction = "";
                }
                ++$i;
            }
            
            return $arrSelectorModels;
        }
        
        /**
         * Performs a selector and returns the element set
         * 
         * @param       string                  $strSelector            The CSS selector
         * @return      XTemplate\ElementList
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function select($strSelector)
        {
            // To safe performance, here is no typecheck
            $strSelector = trim((string) $strSelector, ", ");
            
            // Create the result set
            $objResult = new $this->strElementListClassName();
            
            // Hash the selector
            $strHash = sha1($strSelector);
            
            $arrXPaths = $this->fetchCache($strHash);
            if (!$arrXPaths)
            {
                // We have to create the xpaths
                $arrSelectorModels = $this->createXPath($strSelector);

                // Create the xpaths
                $arrXPaths = array();
                foreach ($arrSelectorModels as $arrModelStack)
                {
                    $arrXPaths[] = implode ("/", $arrModelStack);
                }
                
                // Store them
                $this->storeCache($strHash, $arrXPaths);
            }
            
            // Perform the created selectors
            foreach ($arrXPaths as $strXPath)
            {
                // Perform the xpath
                $objXPathResult =@ $this->objXPath->query($strXPath, $this->objSelectorNode);

                if ($objXPathResult === false)
                {
                    throw new SelectorException("Invalid xpath created: '". $strXPath ."'.");
                }
                
                $objResult->addDOMList($objXPathResult);
            }
            
            return $objResult;
        }
        
        /**
         * Parses the tag name
         * 
         * @param       string              $strTagName
         * @param       string              $strLastConjunction
         * @param       XTemplate\Selectors\SelectorModel   $objModel
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        protected function parseTagName($strTagName, $strLastConjunction, SelectorModel $objModel)
        {
            if ($strTagName === "")
            {
                $strTagName = "*";
            }
            
            // Is there a conjunction?
            if ($strLastConjunction !== "")
            {
                // Yes, Get the selector
                $this->arrSelectorConjunctions[$strLastConjunction]
                        ->performSelector($strLastConjunction, $strTagName, $objModel);
            }
            else
            {
                // No, use the default conjunction
                $objModel->pushPath("descendant::". $strTagName);
            }
            
        }
        
        /**
         * Parses the path selectors. The selector must not be empty. 
         * 
         * @param       string              $strTagName
         * @param       XTemplate\Selectors\SelectorModel   $objModel
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        protected function parsePathSelectors ($strPathSelectors, SelectorModel $objModel)
        {
            $arrMatch = array();
            // Get the single path selectors
            preg_match_all($this->strRegExPath, $strPathSelectors, $arrMatch);
            
            // Walk through the selectors and store the datas in the selectormodel
            $intLength = sizeof($arrMatch[0]);
            $i = 0;
            while ($i < $intLength)
            {
                $this->arrPathSelectors[$arrMatch[2][$i]]
                        ->performSelector($arrMatch[2][$i], $arrMatch[3][$i], $objModel);
                ++$i;
            }
        }
        
        /**
         * Parses the attribute selectors. The selector must not be empty!
         * 
         * @param       string              $strAttribute
         * @param       XTemplate\Selectors\SelectorModel   $objModel
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        protected function parseAttributeSelectors($strAttribute, SelectorModel $objModel)
        {
            $arrMatches = array();
            // Parse the attribute selector
            $varResult = preg_match($this->strRegExAttribute, $strAttribute, $arrMatches);
            
            // Everything alright?
            if ($varResult === 0 || $varResult === false)
            {
                throw new SelectorException("Invalid attribute selector '". $strAttribute ."'.");
            }
            
            // If there is no 6th array entry (using '), use the 5th one (using ")
            if (isset($arrMatches[6]))
            {
                $arrMatches[5] =& $arrMatches[6];
            }
            
            $this->arrAttributeSelectors[$arrMatches[2]]
                    ->performSelector($arrMatches[1], $arrMatches[2], $arrMatches[5], $objModel);
        }
        
        /**
         * Returns a variable from the cache or returns false if it's not in the cache. 
         * 
         * @param       string                  $strKey
         * @return      mixed
         */
        private function fetchCache($strKey)
        {
            // Is the caching extension installed?
            if (!function_exists("apc_fetch"))
            {
                return false;
            }
            
            $varResult = apc_fetch($strKey, $bolResult);
            
            if (!$bolResult)
            {
                return false;
            }
            else
            {
                return $varResult;
            }
        }
        
        /**
         * Stores something in the cache
         * 
         * @param       string                  $strKey
         * @param       mixed                   $varValue
         */
        private function storeCache($strKey, $varValue)
        {
            if (!function_exists("apc_store"))
            {
                return;
            }
            
            apc_store($strKey, $varValue);
        }
    }
}
?>