<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Test
{
    use XTemplate\Engine;
    
    /**
     * This class hosts a new test session. It creates test log and sends them to the host server.
     * 
     * @author      Tobias Pohlen
     * @version     $Id$ 
     * @package     xtemplate
     */
    class Test
    {
        const DIR = "test_logs";
        const PATH_SEPARATOR = '/';
        
        /**
         * Singleton instance
         * @var XTemplate\Test\Test
         */
        private static $objInstance = null;
        
        /**
         * The test logfile
         * @var XTemplate\Core\XML
         */
        private $objLog = null;
        /**
         * The test logfile name
         * @var string
         */
        private $strLogFile = "";
        
        /**
         * Loads the engine class
         */
        public static function loadEngine()
        {
            require_once(dirname(dirname(__FILE__)) . self::PATH_SEPARATOR ."Engine.php");
        }
        
        /**
         * Initializes the test
         */
        private static function init()
        {
            // Set the cache headers
            header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
            header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); 
            header("Cache-Control: no-store, no-cache, must-revalidate"); 
            header("Cache-Control: post-check=0, pre-check=0", false);
            header("Pragma: no-cache");
            
            // Get the correct path separator for the system
            $strPath = dirname(__FILE__);

            self::loadEngine();
        }
        
        /**
         * Singleton implementation
         * 
         * @return      XTemplate\Test\Test
         */
        public static function getInstance()
        {
            if (self::$objInstance === null)
            {
                self::init();
                self::$objInstance = new Test();
            }
            return self::$objInstance;
        }
        
        /**
         * Creates a new Testcase. 
         * 
         * @throws      XTemplate\Test\Exceptions\TestException
         */
        public function __construct()
        {
            // Do not save the file in command line mode
            if (Engine::isInCLIMode())
            {
                return;
            }
            
            // Set the timezone
            date_default_timezone_set("Europe/Berlin");
            
            // Is there an open test?
            if (isset($_GET["test"]))
            {
                $this->strLogFile = $_GET["test"];
            }
            else
            {
                $this->strLogFile = "xtemplate_test_". date("d-m-Y-H-i-s") .".xml";
            }
            
            $this->loadLog();
        }
        
        /**
         * Returns the path to the log file
         * 
         * @return      string
         */
        private function getFilePath()
        {
            return dirname(__FILE__) . self::PATH_SEPARATOR . self::DIR . self::PATH_SEPARATOR . $this->strLogFile;
        }
        
        /**
         * Saves the log
         */
        private function save()
        {
            // Do not save the file in command line mode
            if (Engine::isInCLIMode())
            {
                return;
            }
            
            try 
            {
                $objFile = new \XTemplate\Core\File($this->getFilePath(), "w");
                // Delete the old file
                $objFile->unlink();
                // Save the XML file
                $objFile->write($this->objLog->asXML());
            }
            catch (XTemplate\Core\Exceptions\ConfigException $objException)
            {
                echo $objException->getMessage();
            }
        }
        
        /**
         * Loads the logfile
         * 
         * @throws      XTemplate\Test\Exceptions\TestException
         */
        private function loadLog()
        {
            $bolCreateNew = false;
            if (!file_exists($this->getFilePath()))
            {
                // Create a new logfile
                $this->objLog = new \XTemplate\Core\XML("<testresults />");
                $this->objLog->addChild("logs");
                
                // Add some log infos
                $objInfo = $this->objLog->addChild("info");
                $objInfo->addChild("date", date("d-m-Y-H-i"));
                $objInfo->addChild("php_version", \PHP_VERSION);
                $objInfo->addChild("php_os", \PHP_OS);
                $objInfo->addChild("php_sapi", \PHP_SAPI);
                $this->save();
            }
            else
            {
                $this->objLog = \XTemplate\Core\XML::getInstance($this->getFilePath());
                
                if (!$this->objLog)
                {
                    throw new Exceptions\TestException("Could not load logfile. Please check accessrights for '". self::DIR ."'.");
                }
            }
        }
        
        /**
         * Adds a testcase log
         * 
         * @param       XTemplate\Test\TestCase         $objTestCase
         * @throws      XTemplate\Test\Exceptions\TestException
         */
        public function log(TestCase $objTestCase)
        {
            // Do not log in cli mode
            if (Engine::isInCLIMode())
            {
                return;
            }
            
            $objLog = $this->objLog->logs->addChild("log");
            $objLog->addChild("case_id", $objTestCase->getTestCaseID());
            $objLog->addChild("result", $objTestCase->isSuccessful() ? "true" : "false");
            $objLog->addCDATAChild("comment", $objTestCase->getComment());
            $objLog->addChild("time", $objTestCase->getTime());
            $this->save();
        }
        
        /**
         * Returns the filename
         * 
         * @return      string
         */
        public function getFileName()
        {
            return $this->strLogFile;
        }
        
        /**
         * Returns the log object
         * 
         * @return      \SimpleXMLElement
         */
        public function getLog()
        {
            return $this->objLog;
        }
    }
}
?>
