<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Test
{
    use \XTemplate\Engine;
    
    error_reporting(-1);
    
    /**
     * This class is the base of all test. You define what your output should be like and the rest will be performed
     * automatically. 
     * 
     * @author      Tobias Pohlen
     * @version     $Id$ 
     * @package     xtemplate
     */
    abstract class TestCase
    {
        /**
         * Default result template
         */
        const RESULT = "result.xml";
        /**
         * Description file
         */
        const DESCRIPTION = "description.txt";
        /**
         * Debug enabled
         */
        const DEBUG = false;
        
        /**
         * The test object
         * @var XTemplate\Test\Test
         */
        private $objTest = null;
        /**
         * If the test was successful
         * @var bool
         */
        private $bolSuccessful = false;
        /**
         * A comment for the logfile
         * @var string
         */
        private $strComment = "";
        /**
         * The testcase ID
         * @var string
         */
        private $strTestCaseID = "";
        /**
         * The time the test needed
         * @var float
         */
        private $fltTime = 0.0;
        
        /**
         * Clears the templates from linebreaks, tabs and spaces
         * 
         * @param       string                  $strTemplate
         * @return      string
         */
        protected static function clearTemplate($strTemplate)
        {
            $strTemplate = mb_ereg_replace("\n", "", $strTemplate);
            $strTemplate = mb_ereg_replace("\r", "", $strTemplate);
            $strTemplate = mb_ereg_replace("\t", "", $strTemplate);
            
            // Remove the whitespaces between two tags
            $strTemplate = preg_replace('/>[ ]+</u', "><", $strTemplate);
            return $strTemplate;
        }
        
        /**
         * Creates a new testcase
         */
        public final function __construct()
        {
            $this->objTest = Test::getInstance();
            Test::loadEngine();
            
            // Determine the test case
            $arrClassName = explode('\\', strtolower(get_class($this)));
            $this->setTestCaseID(array_pop($arrClassName));
            
            // Perform the test
            try
            {
                // Load the description file
                $this->strComment .= $this->loadFile(self::DESCRIPTION) . "\n";
                
                $fltTime = microtime(true);
                ob_start();
                $this->bolSuccessful = $this->performTest();
                $strContent = ob_get_clean();
                
                // Shall the content be displayed?
                if (static::DEBUG)
                {
                    echo $strContent;
                }

                // If there is any content from the test method, it shall be added to the comment
                if ($strContent !== "")
                {
                    $this->strComment .= "\n". $strContent;
                }
                $this->fltTime = microtime(true) - $fltTime;
            }
            catch (\Exception $objException)
            {
                $this->bolSuccessful = false;
                $this->setComment("Test unsuccessful with ". get_class($objException) .": '". $objException->getMessage() ."'.");
            }
            
            $this->objTest->log($this);
            
            // If PHP is in command line mode (That means the test is executed via shell), the output shall be "true"
            // or "false"
            if (Engine::isInCLIMode())
            {
                echo $this->bolSuccessful ? "true" : "false";
                exit(0);
            }
        }
        
        /**
         * Sets the comment
         * 
         * @param       string              $strComment
         */
        protected function setComment($strComment)
        {
            $this->strComment = $strComment;
        }
        
        /**
         * Returns the comment
         * 
         * @return      string
         */
        public function getComment()
        {
            return $this->strComment;
        }
        
        /**
         * Returns if the test was successful
         * 
         * @return      bool
         */
        public function isSuccessful()
        {
            return $this->bolSuccessful;
        }
        
        /**
         * Sets the testcase ID
         * 
         * @param       string              $strID
         */
        protected function setTestCaseID($strID)
        {
            $this->strTestCaseID = $strID;
        }

        /**
         * Returns the testcase ID
         * 
         * @return      string
         */
        public function getTestCaseID()
        {
            return $this->strTestCaseID;
        }

        /**
         * Loads the testfile. This is always test.php
         * 
         * @return      string
         */
        protected function loadTestFile()
        {
            ob_start();
            require_once (dirname(__FILE__) . Test::PATH_SEPARATOR ."Cases". Test::PATH_SEPARATOR . $this->strTestCaseID . Test::PATH_SEPARATOR ."test.php");
            return ob_get_clean();
        }
        
        /**
         * Loads a file which perhaps contains the result of the test
         * 
         * @param       string              $strFileName
         * @return      string              Filecontent
         * @throws      XTemplate\Test\Exceptions\TestException
         */
        protected function loadFile($strFileName)
        {
            $strFileName = dirname(__FILE__) . Test::PATH_SEPARATOR ."Cases". Test::PATH_SEPARATOR . 
                    $this->strTestCaseID . Test::PATH_SEPARATOR . $strFileName;
            
            try 
            {
                $objFile = new \XTemplate\Core\File($strFileName, "r");
                $strContent = $objFile->read();
            }
            catch (XTemplate\Core\Exceptions\CoreException $objException)
            {
                throw new Exceptions\TestException($objException);
            }
            
            return $strContent;
        }
        
        /**
         * Returns the time in microseconds
         * 
         * @return      float
         */
        public function getTime()
        {
            return $this->fltTime * 1000;
        }
        
        
        /**
         * Performs the test script and compares the result to a result template
         * 
         * @param       string                  $strResult      The result template file relative to the test dir
         * @return      bool
         */
        protected function performResultTest($strResult = self::RESULT)
        {
            // Get the expected testresult
            try
            {
                $strContent = $this->loadFile($strResult);
            }
            catch (XTemplate\Test\Exceptions\TestException $objException)
            {
                $this->setComment("Could not load result template.");
                return false;
            }
            
            // Load the testfile
            $strResult = $this->loadTestFile();
            
            // Shall the content be displayed?
            if (static::DEBUG)
            {
                echo $strResult;
            }
        
            // Remove the whitespaces, tabs and linebreaks from both results
            return self::clearTemplate($strResult) === self::clearTemplate($strContent);
        }
        
        /**
         * Performs the test
         * 
         * @return      bool
         */
        protected abstract function performTest();
    }
}
?>