<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Test\UnitTests\Core
{
    use \XTemplate\Engine;
    
    $strCurrentDirectory = dirname(__FILE__);
    require_once dirname(dirname(dirname($strCurrentDirectory))) . "/Engine.php";
    
    /**
     * Test for the autoloader class
     * 
     * @author      Tobias Pohlen
     * @package     xtemplate
     * @version     $Id$ 
     */
    class Autoloader extends \PHPUnit_Framework_TestCase
    {
        /**
         * Autoloader
         * @var \XTemplate\Core\Autoloader
         */
        private $objAutoloader = null;
        
        /**
         * Creates the autoloader
         */
        protected function setUp()
        {
            // Create a new Autoloader
            $this->objAutoloader = new \XTemplate\Core\Autoloader();
        }
        
        /**
         * Removes the autoloader again
         */
        protected function tearDown()
        {
            if ($this->objAutoloader !== null)
            {
                unset($this->objAutoloader);
            }
        }

        /**
         * Tests if the autoloader is registrated correctly
         */
        public function testRegistration()
        {
            $arrAutoloaders = spl_autoload_functions();
            
            // Now the autoloader must be registrated
            $this->assertTrue(in_array(array($this->objAutoloader, "autoloadClass"), $arrAutoloaders, true));
        }
        
        /**
         * Tests if the autoloader is unregistrated correctly
         * 
         * @depends testRegistration
         */
        public function testUnregistration()
        {
            // Remove it
            unset($this->objAutoloader);
            
            $arrAutoloaders = spl_autoload_functions();
            
            // Now the autoloader must not be registrated
            $this->assertTrue(!in_array(array($this->objAutoloader, "autoloadClass"), $arrAutoloaders, true));
        }
        
        /**
         * Tests if a class is added to the autoloader correctly
         */
        public function testAddClass()
        {
            // Add the class to the autoloader
            $this->objAutoloader->addClass(
                    "XTemplate\Test\UnitTests\DummyClasses\Dummy1", 
                    Engine::getAbsPath("Test/UnitTests/DummyClasses/Dummy1.php"));
        }
        
        /**
         * Tests if a class can be loaded correctly
         * 
         * @depends testAddClass
         */
        public function testLoadClassExplicitly()
        {
            // The class doesn't exist yet
            $this->assertTrue(!class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy5"));
            
            // Add the class to the autoloader
            $this->objAutoloader->addClass(
                    "XTemplate\Test\UnitTests\DummyClasses\Dummy5", 
                    Engine::getAbsPath("Test/UnitTests/DummyClasses/Dummy5.php"));
            
            // Load the class explicitly
            $this->objAutoloader->autoloadClass("XTemplate\Test\UnitTests\DummyClasses\Dummy5");
            
            // Now the class must exist
            $this->assertTrue(class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy5"));
        }
        
        /**
         * Tests if a class can be loaded correctly
         * 
         * @depends testAddClass
         */
        public function testLoadClassImplicitly()
        {
            // The class doesn't exist yet
            $this->assertTrue(!class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy2"));
            
            // Add the class to the autoloader
            $this->objAutoloader->addClass(
                    "XTemplate\Test\UnitTests\DummyClasses\Dummy2", 
                    Engine::getAbsPath("Test/UnitTests/DummyClasses/Dummy2.php"));
            
            // Now the class must exist since it's loaded implicitly
            $this->assertTrue(class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy2"));
        }
        
        /**
         * Tests if the exception is thrown correctly if the classname is empty
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testAddClassExceptionEmptyClassName()
        {
            $this->objAutoloader->addClass("", "path");
        }
        
        /**
         * Tests if the exception is thrown correctly if the path is empty
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testAddClassExceptionEmptyPath()
        {
            $this->objAutoloader->addClass("classname", "");
        }
        
        /**
         * Tests if a class can be loaded from an XML file
         * 
         * @depends testLoadClassImplicitly
         */
        public function testLoadXML()
        {
            // Create the XML object
            $objXML = new \XTemplate\Core\XML('
                <autoloader>
                    <file class="XTemplate\Test\UnitTests\DummyClasses\Dummy3">Test/UnitTests/DummyClasses/Dummy3.php</file>
                </autoloader>
            ');
            
            // The class must not be available now
            $this->assertTrue(!class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy3"));
            
            $this->objAutoloader->loadXML($objXML);
            
            // The class must be available now
            $this->assertTrue(class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy3"));
        }
        
        /**
         * Tests if a class can be loaded from an XML file with a certain base path
         * 
         * @depends testLoadClassImplicitly
         */
        public function testLoadXMLBasePath()
        {
            // Define the base path
            $strBasePath = Engine::getAbsPath("Test/UnitTests/DummyClasses/");
            
            // Create the XML object
            $objXML = new \XTemplate\Core\XML('
                <autoloader base="'. $strBasePath .'">
                    <file class="XTemplate\Test\UnitTests\DummyClasses\Dummy4">Dummy4.php</file>
                </autoloader>
            ');
            
            // The class must not be available now
            $this->assertTrue(!class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy4"));
            
            $this->objAutoloader->loadXML($objXML);
            
            // The class must be available now
            $this->assertTrue(class_exists("XTemplate\Test\UnitTests\DummyClasses\Dummy4"));            
        }
    }
}
?>