<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Test\UnitTests\Core
{
    use \XTemplate\Engine;
    
    $strCurrentDirectory = dirname(__FILE__);
    require_once dirname(dirname(dirname($strCurrentDirectory))) . "/Engine.php";
    require_once dirname($strCurrentDirectory) . "/DummyClasses/DummyInterpretor.php";
    
    /**
     * Test for the config class. A lot of tests must be done indirectly through the Engine class. 
     * The Core\XML class which is used in this test is basicly the SimpleXMLElement class whith a few addons which are
     * not used in this test. The engine class is only used with two methods. 
     * 
     * @author      Tobias Pohlen
     * @package     xtemplate
     * @version     $Id$ 
     */
    class Config extends \PHPUnit_Framework_TestCase
    {
        /**
         * Engine instance
         * @var XTemplate\Engine
         */
        private $objEngine = null;
        
        /**
         * Creates a new engine instance
         */
        public function setUp()
        {
            $this->objEngine = new \XTemplate\Engine(new \XTemplate\Core\XML('<dummy></dummy>'));
        }
        
        /**
         * Creates a new config instance
         * 
         * @param       string              $strXML         XML string
         * @return      XTemplate\Core\Config
         */
        private function createConfig($strXML)
        {
            $objXML = simplexml_load_string($strXML, \XTemplate\Core\Classes::CORE_XML);
            
            return new \XTemplate\Core\Config(
                    $objXML, 
                    $this->objEngine);
        }
        
        /**
         * Tests if the config is not interpreted if the domain doesn't match
         */
        public function testConfigWrongDomain()
        {
            $objConfig = $this->createConfig('
                <x:config domain="invalid" xmlns:x="http://xtemplate.net/xmlns.html">
                    <x:interpretors name="interpretors">
                        <interpretor name="dummy">dummy</interpretor>
                    </x:interpretors>
                </x:config>
            ');
            
            // Check if the interpretors are registered. 
            $arrRegistered = $objConfig->getInterpretors();
            
            $this->assertTrue(!isset($arrRegistered["dummy"]));
        }
        
        /**
         * Tests if the config is interpreted if the domain matchs
         */
        public function testConfigRightDomain()
        {
            $objConfig = $this->createConfig('
                <x:config domain="'. Engine::getDomain() .'" xmlns:x="http://xtemplate.net/xmlns.html">
                    <x:interpretors name="interpretors">
                        <interpretor name="dummy">dummy</interpretor>
                    </x:interpretors>
                </x:config>
            ');
            
            // Check if the interpretors are registered. 
            $arrRegistered = $objConfig->getInterpretors();
            
            $this->assertTrue(isset($arrRegistered["dummy"]));
        }
        
        /**
         * Tests if you can extend a config file correctly
         */
        public function testConfigExtend()
        {
            global $strCurrentDirectory;
            
            $objConfig = $this->createConfig('
                <x:config extends="'. $strCurrentDirectory .'/files/config.xml" xmlns:x="http://xtemplate.net/xmlns.html"></x:config>
            ');
            
            // Check if the interpretors are registered. 
            $arrRegistered = $objConfig->getInterpretors();
            
            $this->assertTrue(isset($arrRegistered["dummy"]));
        }
        
        /**
         * Tests if registered config interpretors are created correctly. 
         */
        public function testInterpretorInstace()
        {
            // Save how often the interpretor has been created so long
            $intCount = \XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor::getCount();
            
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html">
                    <x:interpretors name="interpretors">
                        <interpretor name="dummy">XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor</interpretor>
                    </x:interpretors>
                    <x:dummy name="dummy" />
                </x:config>
            ');
            
            // Now the counter has to be one more
            $intCount2 = \XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor::getCount();
            
            $this->assertTrue($intCount2 === $intCount + 1);
        }
        
        /**
         * Tests if an exception is thrown if there is not name for an instance of an config entry(Missing name 
         * attribute)
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testAddConfigEntryMissingName()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html"></x:config>
            ');
            
            // Create an interpretor and add it
            $objConfig->addConfigEntry(null, 
                    new \XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor(
                            new \XTemplate\Core\XML("<dummy></dummy>"), 
                            $objConfig));
        }
        
        /**
         * Tests if a certain config entry can be received from the config
         * 
         * @depends testInterpretorInstace
         */
        public function testReceiveInterpretor()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html">
                    <x:interpretors name="interpretors">
                        <interpretor name="dummy">XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor</interpretor>
                    </x:interpretors>
                    <x:dummy name="dummy" />
                </x:config>
            ');
            
            $this->assertTrue(
                    $objConfig->getConfigEntry("dummy") instanceof \XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor);
        }
        
        /**
         * Tests if an exception is thrown when you ask for a not existing config entry
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testReceiveNotExistingInterpretor()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html"></x:config>
            ');
            
            $objConfig->getConfigEntry("dummy");
        }
        
        /**
         * Tests if the collection of all instances of a certain interpretor type is correct.
         */
        public function testGetCollectionEmpty()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html"></x:config>
            ');
            
            $this->assertEquals(array(), $objConfig->getConfigEntries("dummy"));
        }
        
        /**
         * Tests if the collection of all instances of a certain interpretor type is correct.
         */
        public function testGetCollection()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html">
                    <x:interpretors name="interpretors">
                        <interpretor name="dummy">XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor</interpretor>
                    </x:interpretors>
                    <x:dummy name="dummy" />
                    <x:dummy name="dummy2" />
                </x:config>
            ');
            
            $this->assertEquals(2, count($objConfig->getConfigEntries("dummy")));
        }
        
        /**
         * Tests the array interface: offsetExists
         */
        public function testArrayIntefaceOffsetExistsTrue()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html">
                    <x:interpretors name="interpretors">
                        <interpretor name="dummy">XTemplate\Test\UnitTests\DummyClasses\DummyInterpretor</interpretor>
                    </x:interpretors>
                    <x:dummy name="dummy" />
                </x:config>
            ');
            
            $this->assertTrue(isset($objConfig["dummy"]));
        }
        
        /**
         * Tests the array interface: offsetExists
         */
        public function testArrayIntefaceOffsetExistsFalse()
        {
            $objConfig = $this->createConfig('
                <x:config xmlns:x="http://xtemplate.net/xmlns.html"></x:config>
            ');
            
            $this->assertTrue(!isset($objConfig["dummy"]));
        }
    }
}
?>