<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Test\UnitTests\Core
{
    use \XTemplate\Engine;
    
    $strCurrentDirectory = dirname(__FILE__);
    require_once dirname(dirname(dirname($strCurrentDirectory))) . "/Engine.php";
    
    /**
     * Test for the RAII file implementation which is used to work with files. This test might be a bit slow since it
     * works with the file system.
     * 
     * @author      Tobias Pohlen
     * @package     xtemplate
     * @version     $Id$ 
     */
    class File extends \PHPUnit_Framework_TestCase
    {
        /**
         * Working directory
         * @var string
         */
        private $strDir = "";
        /**
         * Test file 1
         * @var string 
         */
        private $strTest = "";
        /**
         * Test file 2
         * @var string 
         */
        private $strEmpty = "";
        
        /**
         * Creates a working directory
         * 
         * @throws XTemplate\Test\Exceptions\TestException
         */
        public function setUp()
        {
            global $strCurrentDirectory;
            
            $this->strTest = $strCurrentDirectory . "/files/test.txt";
            $this->strEmpty = $strCurrentDirectory . "/files/empty.txt";
            $this->strDir = $strCurrentDirectory . "/working/";
            
            if (!@mkdir($this->strDir))
            {
                throw new \XTemplate\Test\Exceptions\TestException("Could not create working directory.");
            }
            @chmod($this->strDir, 0777);
        }
        
        /**
         * Removes the working directoy
         * 
         * @throws XTemplate\Test\Exceptions\TestException
         */
        public function tearDown()
        {
            // Delete all files of the directory
            $hdlDir = @ opendir($this->strDir);
            if (!$hdlDir)
            {
                throw new \XTemplate\Test\Exceptions\TestException("Could not open working directory.");
            }
            
            while ($strFile = readdir($hdlDir))
            {
                if ($strFile !== "." && $strFile !== "..")
                {
                    // Delete the file
                    if (!@unlink($this->strDir . $strFile))
                    {
                        throw new \XTemplate\Test\Exceptions\TestException("Could not delete file '". $strFile ."'.");
                    }
                }
            }
            
            // Delete the directory
            if (!@rmdir($this->strDir))
            {
                throw new \XTemplate\Test\Exceptions\TestException("Could not delete working directory.");
            }
        }
        
        /**
         * Tests if a file can be opened and closed correctly in mode R
         */
        public function testFOpenR()
        {
            new \XTemplate\Core\File($this->strTest, "r");
        }
        
        /**
         * Tests if a file can be opened and closed correctly in mode R
         */
        public function testFOpenW()
        {
            new \XTemplate\Core\File($this->strDir . "test.txt", "w");
        }
        
        /**
         * You cannot open a file to read that doesn't exist.
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testFReadNotExisting()
        {
            $objFile = new \XTemplate\Core\File($this->strDir . "test.txt", "r");
        }
        
        /**
         * Tests if you can read from a file correctly which is empty.
         */
        public function testFReadEmpty()
        {
            global $strCurrentDirectory;
            $objFile = new \XTemplate\Core\File($this->strEmpty, "r");
            $this->assertEmpty($objFile->read());
        }
        
        /**
         * Tests if you can read from a file the first char correctly. 
         */
        public function testFReadLength()
        {
            $objFile = new \XTemplate\Core\File($this->strTest, "r");
            $this->assertEquals("test4711", $objFile->read(8));
        }
        
        /**
         * Tests if you can read from a file more char than it actually has. 
         */
        public function testFReadLengthTooLong()
        {
            $objFile = new \XTemplate\Core\File($this->strTest, "r");
            $this->assertEquals("test4711herewego", $objFile->read(800));
        }
        
        /**
         * Tests if you can read the whole content from a file.
         */
        public function testFReadLengthAuto()
        {
            $objFile = new \XTemplate\Core\File($this->strTest, "r");
            $this->assertEquals("test4711herewego", $objFile->read());
        }
        
        /**
         * Tests if you can read the file in write mode
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testFReadWriteMode()
        {
            // Create a file
            $objFile = new \XTemplate\Core\File($this->strDir . "test.txt", "w");
            $objFile->write("asdf");
            
            $this->assertTrue($objFile->exists());
            
            $objFile = new \XTemplate\Core\File($this->strDir . "test.txt", "w+");
            $objFile->write("asdf");
            $objFile->read();
        }
        
        /**
         * Tests if you can write to a file
         */
        public function testFWrite()
        {
            $objFile = new \XTemplate\Core\File($this->strDir . "test.txt", "w");
            $objFile->write("test4711");
        }
        
        /**
         * Tests if you can write to a file in read mode
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testFWriteReadMode()
        {
            $objFile = new \XTemplate\Core\File($this->strDir . "test.txt", "r");
            $objFile->write("test4711");
        }
        
        /**
         * Tests if you delete files
         * 
         * @depends testFWrite
         */
        public function testUnlink()
        {
            $objFile = new \XTemplate\Core\File($this->strDir . "test.txt", "w");
            $objFile->write("test4711");
            
            $this->assertTrue($objFile->exists());
            
            $objFile->unlink();
            
            unset($objFile);
            
            $this->assertTrue(file_exists($this->strDir . "test.txt"));
        }
    }
}
?>