<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate\Test\UnitTests\Core
{
    use \XTemplate\Engine;
    
    $strCurrentDirectory = dirname(__FILE__);
    require_once dirname(dirname(dirname($strCurrentDirectory))) . "/Engine.php";
    
    /**
     * Test for the SimpleXML extension of the engine.
     * 
     * @author      Tobias Pohlen
     * @package     xtemplate
     * @version     $Id$ 
     */
    class XML extends \PHPUnit_Framework_TestCase
    {
        /**
         * Tests if we can create an XML object from an XML string. 
         */
        public function testXMLStringLegal()
        {
            $objXML = \XTemplate\Core\XML::getInstance('<test></test>');
            $this->assertTrue($objXML instanceof \XTemplate\Core\XML);
        }
        
        /**
         * Tests if we can create an XML object from an XML string which is illegal xml. 
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testXMLStringIllegal()
        {
            \XTemplate\Core\XML::getInstance('<test>test>');
        }
        
        /**
         * Tests if XML objectes are passed through the class correctly.
         */
        public function testXMLObject()
        {
            $objXML = new \XTemplate\Core\XML("<test></test>");
            $objXML2 = \XTemplate\Core\XML::getInstance($objXML);
            
            $this->assertTrue($objXML === $objXML2);
        }
        
        /**
         * Tests if you can load an xml file from the filesystem correctly.
         */
        public function testXMLFileLegal()
        {
            global $strCurrentDirectory;
            
            $objXML = \XTemplate\Core\XML::getInstance($strCurrentDirectory . "/files/config.xml");
            $this->assertTrue($objXML instanceof \XTemplate\Core\XML);
        }
        
        /**
         * Tests if an exception is thrown when the file is invalid XML
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testXMLFileIllegal()
        {
            global $strCurrentDirectory;
            
            \XTemplate\Core\XML::getInstance($strCurrentDirectory . "files/test.txt");
        }
        
        /**
         * Tests if an exception is thrown when the file is cannot be loaded. 
         * 
         * @expectedException XTemplate\Core\Exceptions\CoreException
         */
        public function testXMLFileIllegal2()
        {
            \XTemplate\Core\XML::getInstance("illegal");
        }
        
        /**
         * Tests if you can read attributes which exist. 
         * 
         * @depends testXMLStringLegal
         */
        public function testGetAttribute()
        {
            $objXML = \XTemplate\Core\XML::getInstance('<dummy><test name="value" /></dummy>');
            $strAttribute = $objXML->test->getAttribute("name");
            
            $this->assertEquals("value", $strAttribute);
        }
        
        /**
         * Tests if the access to not existing attributes returns null. 
         * 
         * @depends testXMLStringLegal
         */
        public function testGetAttributeNull()
        {
            $objXML = \XTemplate\Core\XML::getInstance('<dummy><test name="value" /></dummy>');
            $strAttribute = $objXML->getAttribute("name2");
            
            $this->assertTrue($strAttribute === null);
        }
        
        /**
         * Tests if you can add CDATA children correctly.  
         * 
         * @depends testXMLStringLegal
         */
        public function testAddCDATAChild()
        {
            $objXML = \XTemplate\Core\XML::getInstance('<dummy></dummy>');
            $objXML->addCDataChild("test", "<hello>");
            
            $this->assertEquals("<hello>", (string) $objXML->test);
        }
        
        /**
         * Tests if you can convert your XML structure to an array correctly.
         */
        public function testToArray()
        {
            $objXML = \XTemplate\Core\XML::getInstance(
                    '<dummy><test1>test</test1><test2>test</test2><test3><test>test</test></test3></dummy>');
            
            $this->assertEquals(array(
                "test1" => "test", 
                "test2" => "test", 
                "test3" => array(
                    "test" => "test"
                )
            ), $objXML->toArray());
        }
    }
}
?>