<?php
/**
 * x:template - PHP based template engine
 * Copyright (c) 2011 - 2012 by Tobias Pohlen <tobias.pohlen@xtemplate.net>
 * 
 * Released under the GPL License.
 */
namespace XTemplate
{
    use \XTemplate\Core\Params;
    use \ArrayAccess;
    use \XTemplate\Core\Classes;
    use \XTemplate\Core\Constants;
    use \XTemplate\Core\File;
    use \XTemplate\Core\Helper;
    
    
    /**
     * This is the base class for all view classes. You extend the this class and overwrite the render method. In this 
     * method, you transfer the datas from your model datas to the template. You can add any model data by the assign
     * method. 
     * 
     * @author      Tobias Pohlen
     * @package     xtemplate
     * @version     $Id$ 
     */
    abstract class ViewBase extends \DOMDocument implements ArrayAccess, Core\Interfaces\EnginePart, Interfaces\Selector, 
                                                        Interfaces\SubView
    {
        const DOM_NODE      = "DOMNode";
        const DOM_DOCUMENT  = "DOMDocument";
        const DOM_ELEMENT   = "DOMElement";
        const TYPE          = 0;
        const DEFAULT_VALUE = 1;
        
        /**
         * If this constant is set to true, an exception is thrown when an undefined model is assined or the model type
         * is wrong
         */
        const STRICT_MODE   = true;
        
        /**
         * Model interfaces for the different view classes
         * @var array
         */
        private $arrModelInterfaces = array();
        
        /**
         * Connected engine instance
         * @see XTemplate\Core\Interfaces\EnginePart
         * @var XTemplate\Engine
         */
        private $objEngine = null;
        /**
         * Postparsers
         * @var array
         */
        private $arrPostParsers = array();
        /**
         * Preparsers
         * @var array
         */
        private $arrPreParsers = array();
        /**
         * Assigned models
         * @var array
         */
        private $arrModels = array();
        /**
         * Toplevel section
         * @var XTemplate\Section
         */
        private $objTopLevelSection = null;
        /**
         * Working section
         * @var XTemplate\Section
         */
        private $objWorkingSection = null;
        /**
         * Whether the template was already rendered
         * @var bool
         */
        private $bolRendered = false;
        
        /**
         * Creates a new template engine
         * 
         * @param      string               $strXMLFile         The path to an xml file or a plane XML string
         * @param      XTemplate\Engine     $objEngine          The connected engine instance. 
         * @throws     XTemplate\Parsers\Exceptions\ParsingException
         * @throws     XTemplate\Exceptions\TemplateException
         */
        public function __construct($strXMLFile, Engine $objEngine = null)
        {
            // Load the default models
            $this->loadDefaultModels();
            
            // Get the correct engine instance
            $this->setEngineInstance($objEngine);
            
            $objEngine = $this->getEngineInstance();
            
            parent::__construct(Constants::DOM_VERSION, Constants::DOM_CHARSET);
            
            // Register the dom class overloads to get own functionality 
            $this->registerNodeClass(self::DOM_DOCUMENT, $objEngine->getClassName(self::DOM_DOCUMENT));
            $this->registerNodeClass(self::DOM_NODE, $objEngine->getClassName(self::DOM_NODE));
            $this->registerNodeClass(self::DOM_ELEMENT, $objEngine->getClassName(self::DOM_ELEMENT));
            
            // Get all parsers from the config
            $this->importParsers();
            
            // Load the template
            $this->loadTemplate($strXMLFile);
        }
        
        /**
         * Imports the pre and post parsers from the configuration
         */
        private function importParsers()
        {
            $arrConfigEntries = $this->getEngineInstance()->getConfig()->getConfigEntries("parsers");
            foreach ($arrConfigEntries as $objConfigParser)
            {
                // Are there any preparsers?
                if (count($objConfigParser->getPreParsers()) > 0)
                {
                    $this->arrPreParsers = array_merge($this->arrPreParsers, $objConfigParser->getPreParsers());
                }

                // Are there any postparsers?
                if (count($objConfigParser->getPostParsers()) > 0)
                {
                    $this->arrPostParsers = array_merge($this->arrPostParsers, $objConfigParser->getPostParsers());
                }
            }
        }
        
        /**
         * Loads the template
         * 
         * @param       string              $strTemplate            This is a filename or a XML string
         * @throws      XTemplate\Exceptions\TemplateException
         */
        public function loadTemplate($strTemplate)
        {
            Params::string($strTemplate, __CLASS__, __FUNCTION__);
            
            // Is there a file like this?
            if (file_exists($strTemplate))
            {
                // Get the file content
                try
                {
                    $objFile = new File($strTemplate, "r");
                    // Read the file content
                    $strContent = $objFile->read();
                }
                catch (Core\Exceptions\ConfigException $objException)
                {
                    throw new Exceptions\TemplateException($objException);
                }
            }
            else
            {
                // There is no such file, use the input as XML string
                $strContent = $strTemplate;
            }
            
            if (!@$this->_loadHTML($strContent))
            {
                throw new Exceptions\TemplateException("Could not parse template.");
            }
            
            // Perform the preparsers
            $this->performParsers($this->arrPreParsers);
            
            
            // Create the toplevel section of the template
            $this->objTopLevelSection = $this->getEngineInstance()->createInstance(Classes::SECTION, $this->documentElement, $this);
            $this->objWorkingSection  = clone $this->objTopLevelSection;
        }
        
        /**
         * This is a wrapper method for the native loadHTML method. Since we cannot overwrite these and still be 
         * compatible to PHP 5.3 and PHP 5.4. 
         * 
         * @param       string          $strTemplate        The template string
         * @param       int             $intOptions         Flags
         * @return      bool
         */
        protected function _loadHTML($strTemplate)
        {
            return $this->loadHTML($strTemplate);
        }
        
        /**
         * Creates the HTML code and removes the <remove>-tags which are used to wrap some contents
         * 
         * @return      string
         */
        public function saveHTML()
        {
            return preg_replace('/<[\/]?remove[^>]*>/', "", parent::saveHTML());
        }
        
        /**
         * Loads the predefined model interface into the current object
         * 
         * @throws      XTemplate\Exceptions\ModelException
         */
        private function loadDefaultModels()
        {
            // Define the interface
            $this->defineModelInterface();
            
            foreach ($this->arrModelInterfaces as $strName => $arrMapping)
            {
                $this->assign($strName, $arrMapping[self::DEFAULT_VALUE]);
            }
        }
        
        /**
         * Sets the engine instance
         * 
         * @param       XTemplate\Engine            $objEngine      The engine instance
         * @throws      CoreException
         */
        public function setEngineInstance(Engine $objEngine = null)
        {
            // You cannot overwrite an existing engine instance
            if ($this->objEngine !== null)
            {
                throw new Exceptions\CoreException("You cannot change the engine instance of an existing object.");
            }
            
            // If there is no engine instance given, select the default instance
            if ($objEngine === null)
            {
                $objEngine = Engine::getDefaultInstance();
            }
            
            $this->objEngine = $objEngine;
        }

        /**
         * Performs the pre or post parsers
         * 
         * @param       array               $arrParsers         The list of parsers
         * @throws      XTemplate\Parsers\Exceptions\ParsingException
         */
        private function performParsers(array $arrParsers)
        {
            $objEngine = $this->getEngineInstance();
            
            // Walk through the parser list
            foreach ($arrParsers as $strParserClass)
            {
                // Check for any overloads
                $objParser = $objEngine->createInstance($strParserClass, $this);
                
                // Check the class
                if (!$objParser instanceof Parsers\Base)
                {
                    throw new Parsers\Exceptions\ParsingException($strParserClass . " is not a valid parser.");
                }
            }
        }
        
        /**
         * Renders the template and returns the content HTML as string
         * 
         * @throws      XTemplate\Exceptions\RenderException
         */
        public function render()
        {
            // Do not render the template twice
            if ($this->bolRendered)
            {
                return;
            }
            
            // Perform the default rendering method
            $this->_render();
            
            // Render the sections
            $this->objTopLevelSection->render();
            
            // Perform the post parser
            $this->performParsers($this->arrPostParsers);
            $this->bolRendered = true;
        }
        
        /**
         * Returns the template as HTML
         * 
         * @return      string
         */
        public function __toString()
        {
            // Since this method must not throw any exception, they are cought here and the error message will be
            // returned
            try
            {
                // The template must be rendered
                if (!$this->bolRendered)
                {
                    $this->render();
                }
                return $this->saveHTML();
            }
            catch (\Exception $objError)
            {
                return $objError->getMessage();
            }
        }
        

        /**
         * A abbreviation of the getSectionClone method. 
         * 
         * @param       string              $strSectionName     The name of the section which shall be cloned
         * @return      XTemplate\Section
         * @throws      OutOfRangeException
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        public function __invoke()
        {
            $this->checkRendered();
            $arrArgs = func_get_args();
            
            // Are the argument correct?
            if (count($arrArgs) < 1)
            {
                throw new Exceptions\SectionizeException("Missing argument: section name.");
            }
            
            // Get the section
            return $this->getSectionClone($arrArgs[0]);
        }
        
        /**
         * Assignes a model to the template
         * 
         * @param       string              $strName
         * @param       mixed               $varModel
         * @return      XTemplate\ViewBase
         * @throws      XTemplate\Exceptions\ModelException
         */
        public function assign($strName, $varModel)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            
            // Is this class in strict mode?
            if (static::STRICT_MODE)
            {
                // Check the model definition
                if (!isset($this->arrModelInterfaces[$strName]))
                {
                    throw new Exceptions\ModelException("Undefined model '". $strName ."'.");
                }
                
                $varModel = Params::checkType($varModel, $this->arrModelInterfaces[$strName][self::TYPE]);
            }
            
            $this->arrModels[$strName] = $varModel;
            return $this;
        }
        
        /**
         * This class defines a model interface
         * 
         * @param       string                  $strName        The name of the model
         * @param       string                  $strType        The type of the model. If the type is no primary type, 
         *                                                      it's assumed as a class name. You can find constants for
         *                                                      the primary types in XTemplate\Core\Constants
         * @param       mixed                   $varDefault     Default value
         */
        protected function defModel($strName, $strType, $varDefault = null)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            Params::string($strType, __CLASS__, __FUNCTION__);
            
            
            // Add the interface definitions
            $this->arrModelInterfaces[$strName] = array(
                self::TYPE          => $strType, 
                self::DEFAULT_VALUE => $varDefault
            );
        }
        
        /**
         * Adds a postparser
         * 
         * @param       string              $strClassName
         */
        public function addPostParser($strClassName)
        {
            Params::string($strClassName, __CLASS__, __FUNCTION__);
            
            $this->arrPostParsers[] = $strClassName;
        }
        
        /**
         * Assignes a subview
         * 
         * @param       string              $strName        The name of the subview
         * @param       XTemplate\SubView   $objDoc         The subview document
         */
        public function assignSubView($strName, \XTemplate\SubView $objDoc)
        {
            $this->objWorkingSection->assignSubView($strName, $objDoc);
        }
        
        /**
         * Returns the subviews assigned under a certain name. If there are no subviews an empty object storage is 
         * returned.
         * 
         * @param       string              $strName        The name of the subview
         * @return      XTemplate\SubViewStorage
         */
        public function getAssignedSubViews($strName)
        {
            return $this->objWorkingSection->getAssignedSubViews($strName);
        }
        
        
        /**
         * Appends the subviews to a certain element in the template. e.g.:
         * You add multiple subviews in your controller to the view under the name "content". Now you want to put these
         * views to the element with the ID content:
         * 
         * $this->attachSubViews("#content", "content");
         * 
         * @param       string              $strSelector    A CSS selector which finds the corresponding nodes
         * @param       string              $strSubViewName The name of the subviews which shall be attached
         */
        public function attachSubViews($strSelector, $strSubViewName)
        {
            $this->objWorkingSection->attachSubViews($strSelector, $strSubViewName);
        }
        
        /**
         * This method determines the correct path for the comfort view classes
         * 
         * @param       string              $strInputPath
         * @return      string
         */
        protected function getComfortPath($strInputPath)
        {
            Params::string($strPath, __CLASS__, __FUNCTION__);
            
            // Determine the path to the template file
            $objRefClass = new \ReflectionClass($this);
            $strFolder = dirname($objRefClass->getFileName()) . Engine::PATH_SEPARATOR;
            return Helper::createRelPath($strFolder, $strInputPath);
        }
        
        /**
         * Returns assigned datas from the name
         * 
         * @param       string              $strName
         * @return      mixed
         * @throws      XTemplate\Exceptions\RenderException
         */
        protected function getDatas($strName)
        {
            Params::string($strName, __CLASS__, __FUNCTION__);
            
            // Are there datas with this name?
            if (!isset($this->arrModels[$strName]))
            {
                throw new Exceptions\RenderException("No datas with name '". $strName ."' assigned.");
            }
            
            return $this->arrModels[$strName];
        }
        
        /**
         * Returns the document element
         * 
         * @return      DOMElement
         */
        public function getDocumentElement()
        {
            return $this->documentElement;
        }
        
        /**
         * Returns the configuration for this instance
         * 
         * @return      XTemplate\Core\Config
         */
        protected function getConfig()
        {
            return $this->getEngineInstance()->getConfig();
        }

        /**
         * Returns the working section
         * 
         * @return      XTemplate\Section
         */
        protected function getWorkingSection()
        {
            return $this->objWorkingSection;
        }
        
        /**
         * Returns the engine instance
         * 
         * @return      XTemplate\Engine
         * @throws      CoreException
         */
        public function getEngineInstance()
        {
            return $this->objEngine;
        }
        
        /**
         * Returns the filename
         * 
         * @return      string
         */
        public function getFileName()
        {
            return $this->documentURI;
        }
        
        /**
         * Returns a section 
         * 
         * @param       string              $strSectionName     The name of the section which shall be cloned
         * @return      XTemplate\Section
         * @throws      OutOfRangeException
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        public function getSectionClone($strSectionName)
        {
            $this->checkRendered();
            return $this->objWorkingSection->getSectionClone($strSectionName);
        }
        
        /**
         * Returns a clone of the working section
         * 
         * @return      XTemplate\Section
         * @throws      OutOfRangeException
         * @throws      XTemplate\Exceptions\SectionizeException
         */
        public function getClone()
        {
            return clone $this->objWorkingSection;
        }
     
        /**
         * Returns if there are element which match this selector
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @return      bool
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetExists($strSelector)
        {
            $this->checkRendered();
            return $this->objWorkingSection->offsetExists($strSelector);
        }
        
        /**
         * Sets the content of the matched elements
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @param       string              $strContent
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetSet($strSelector, $strContent)
        {
            $this->checkRendered();
            return $this->objWorkingSection->offsetSet($strSelector, $strContent);
        }
        
        /**
         * Returns the resultset of the selector
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @return      XTemplate\ElementList
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetGet($strSelector)
        {
            $this->checkRendered();
            return $this->objWorkingSection->offsetGet($strSelector);
        }
        
        /**
         * Removes the matching elements from the template
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function offsetUnset($strSelector)
        {
            $this->checkRendered();
            return $this->objWorkingSection->offsetUnset($strSelector);
        }
        
        /**
         * Returns the resultset of the selector
         * 
         * @see         \ArrayAccess
         * @param       string              $strSelector
         * @return      XTemplate\ElementList
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        public function select($strSelector)
        {
            $this->checkRendered();
            return $this->objWorkingSection->select($strSelector);
        }
        
        /**
         * Checks if the template was already rendered. If it was rendered, you cannot modify the template anymore. 
         * 
         * @throws      XTemplate\Selectors\Exceptions\SelectorException
         */
        private function checkRendered()
        {
            if ($this->bolRendered)
            {
                throw new Selectors\Exceptions\SelectorException("The template was already rendered.");
            }
        }
        
        /**
         * This method can be overwritten to define the model interface
         */
        protected function defineModelInterface() {}
        
        /**
         * Custom rendering method. This method has to be implemented from the final view class developer. This method
         * is automatically called when the template is rendered. 
         * 
         * @throws      XTemplate\Exceptions\RenderException
         */
        protected abstract function _render();
    }
}
?>